from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import Client
from ...models.verify_password_json_body import VerifyPasswordJsonBody
from ...models.verify_password_response_200 import VerifyPasswordResponse200
from ...livtypes import Response


def _get_kwargs(
    user_id: str,
    *,
    client: Client,
    json_body: VerifyPasswordJsonBody,
) -> Dict[str, Any]:
    url = "{}/users/{user_id}/verify_password".format(client.base_url, user_id=user_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "json": json_json_body,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Union[Any, VerifyPasswordResponse200]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = VerifyPasswordResponse200.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = cast(Any, None)
        return response_422
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = cast(Any, None)
        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Union[Any, VerifyPasswordResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    user_id: str,
    *,
    client: Client,
    json_body: VerifyPasswordJsonBody,
) -> Response[Union[Any, VerifyPasswordResponse200]]:
    """Verify the password of a user

     Check that the user's password matches the supplied input.
    Useful for custom auth flows and re-verification.

    Args:
        user_id (str):
        json_body (VerifyPasswordJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, VerifyPasswordResponse200]]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    user_id: str,
    *,
    client: Client,
    json_body: VerifyPasswordJsonBody,
) -> Optional[Union[Any, VerifyPasswordResponse200]]:
    """Verify the password of a user

     Check that the user's password matches the supplied input.
    Useful for custom auth flows and re-verification.

    Args:
        user_id (str):
        json_body (VerifyPasswordJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, VerifyPasswordResponse200]
    """

    return sync_detailed(
        user_id=user_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    user_id: str,
    *,
    client: Client,
    json_body: VerifyPasswordJsonBody,
) -> Response[Union[Any, VerifyPasswordResponse200]]:
    """Verify the password of a user

     Check that the user's password matches the supplied input.
    Useful for custom auth flows and re-verification.

    Args:
        user_id (str):
        json_body (VerifyPasswordJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, VerifyPasswordResponse200]]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    user_id: str,
    *,
    client: Client,
    json_body: VerifyPasswordJsonBody,
) -> Optional[Union[Any, VerifyPasswordResponse200]]:
    """Verify the password of a user

     Check that the user's password matches the supplied input.
    Useful for custom auth flows and re-verification.

    Args:
        user_id (str):
        json_body (VerifyPasswordJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, VerifyPasswordResponse200]
    """

    return (
        await asyncio_detailed(
            user_id=user_id,
            client=client,
            json_body=json_body,
        )
    ).parsed
